import java.io.*;
import java.util.*;

public class SolMainJava {
    static final int INF = 1000000000;
    static int n, m, s, t;
    static ArrayList<Integer>[] adj;    // graph: for each vertex, its neighbors
    static int[] dis;                   // distances from each vertex to t (destination)
    static int[] bedge;                 // for each vertex v, a chosen neighbor u with dis[v]==dis[u]+1
    static ArrayList<Integer>[] children; // tree defined by bedge: for each v, children[v] = { u such that bedge[u]==v }
    
    // DSU arrays for union–find (used during merging of candidate sets)
    static int[] dsuParent, dsuRank;
    
    // For each vertex v, alt[v] holds candidate pairs (key, vertex)
    // (The key is an integer value; the set is maintained in increasing order.)
    static TreeSet<Pair>[] alt;
    // modVal[v] is an offset value for the keys stored in alt[v]
    static int[] modVal;
    // g[v] is computed in the calc_g procedure
    static int[] g;
    
    // f[v] is computed later using a Dijkstra–like method (calc_f)
    static int[] f;
    
    public static void main(String[] args) throws IOException {
        FastReader in = new FastReader();
        PrintWriter out = new PrintWriter(System.out);
 
        n = in.nextInt();
        m = in.nextInt();
        s = 0;         // Lisbon is vertex 0 (since input vertices are 1-indexed)
        t = n - 1;     // Porto is vertex n-1
 
        // Build the graph (0-indexed)
        adj = new ArrayList[n];
        for (int i = 0; i < n; i++) {
            adj[i] = new ArrayList<>();
        }
        for (int i = 0; i < m; i++) {
            int u = in.nextInt() - 1;
            int v = in.nextInt() - 1;
            adj[u].add(v);
            adj[v].add(u);
        }
 
        // Run BFS from t to compute distances from each vertex to t.
        dis = new int[n];
        Arrays.fill(dis, -1);
        ArrayDeque<Integer> dq = new ArrayDeque<>();
        dq.add(t);
        dis[t] = 0;
        while (!dq.isEmpty()){
            int cur = dq.poll();
            for (int nb : adj[cur]) {
                if (dis[nb] == -1) {
                    dis[nb] = dis[cur] + 1;
                    dq.add(nb);
                }
            }
        }
 
        // For each vertex, choose a neighbor that is one step closer to t.
        bedge = new int[n];
        Arrays.fill(bedge, -1);
        for (int v = 0; v < n; v++) {
            for (int nb : adj[v]) {
                if (dis[v] == dis[nb] + 1) {
                    bedge[v] = nb;
                    break;
                }
            }
        }
 
        // Build the tree defined by bedge: for every vertex v (except t), add v as a child of bedge[v].
        children = new ArrayList[n];
        for (int i = 0; i < n; i++){
            children[i] = new ArrayList<>();
        }
        for (int v = 0; v < n; v++){
            if(bedge[v] != -1){
                children[bedge[v]].add(v);
            }
        }
 
        // Initialize DSU arrays.
        dsuParent = new int[n];
        dsuRank = new int[n];
        for (int i = 0; i < n; i++){
            dsuParent[i] = i;
            dsuRank[i] = 0;
        }
 
        // Prepare arrays for calc_g.
        alt = new TreeSet[n];
        modVal = new int[n];
        g = new int[n];
        for (int i = 0; i < n; i++){
            g[i] = INF;
        }
 
        // Do a postorder traversal of the bedge–tree.
        // We first compute an ordering by a DFS (iterative, using a stack) then reverse it.
        int[] order = new int[n];
        int orderSize = 0;
        Stack<Integer> stack = new Stack<>();
        stack.push(t);
        while (!stack.isEmpty()){
            int cur = stack.pop();
            order[orderSize++] = cur;
            for (int child : children[cur]) {
                stack.push(child);
            }
        }
        // Reverse to get postorder: children come before their parent.
        for (int i = 0; i < orderSize / 2; i++){
            int tmp = order[i];
            order[i] = order[orderSize - 1 - i];
            order[orderSize - 1 - i] = tmp;
        }
 
        // Process vertices in postorder to compute g values.
        // This mimics the recursive calc_g() but uses an iterative loop.
        for (int idx = 0; idx < orderSize; idx++){
            int v = order[idx];
            // Initialize alt[v] as an empty set.
            alt[v] = new TreeSet<>();
            // For every neighbor nb of v, if nb is not "linked" by the tree edge in either direction,
            // add candidate (1 + dis[nb], nb) into alt[v].
            for (int nb : adj[v]) {
                if (bedge[nb] == v || (bedge[v] == nb)) continue;
                alt[v].add(new Pair(1 + dis[nb], nb));
            }
            modVal[v] = 0;
            // For each child of v (in the tree defined by bedge), merge its candidate set into v.
            for (int child : children[v]) {
                int childMod = modVal[child] + 1;  // similar to: modu = calc_g(child) + 1
                mergeAlt(v, child, childMod);
                unionDSU(v, child);
            }
 
            // Remove any candidate whose vertex is now in the same DSU component as v.
            while (!alt[v].isEmpty()){
                Pair candidate = alt[v].first();
                if(findDSU(candidate.second) == findDSU(v)){
                    alt[v].pollFirst();
                } else {
                    break;
                }
            }
            if (!alt[v].isEmpty()){
                g[v] = alt[v].first().first + modVal[v];
            }
            // If alt[v] is empty, g[v] remains INF.
        }
 
        // Now, compute f[] using a Dijkstra-like approach.
        // f[v] = max( min_{w in adj[v]} (1+f[w]), g[v] )
        f = new int[n];
        Arrays.fill(f, INF);
        boolean[] fvis = new boolean[n];
        PriorityQueue<Pair> pq = new PriorityQueue<>();
        f[t] = 0;
        pq.add(new Pair(0, t));
 
        while(!pq.isEmpty()){
            Pair cur = pq.poll();
            int v = cur.second;
            if(fvis[v]) continue;
            fvis[v] = true;
 
            if(v != t){
                int best = INF;
                for (int nb : adj[v]) {
                    best = Math.min(best, 1 + f[nb]);
                }
                f[v] = Math.max(best, g[v]);
            } else {
                f[v] = 0;
            }
 
            if(v == s) break;
 
            for (int nb : adj[v]){
                if(!fvis[nb]){
                    pq.add(new Pair(f[v], nb));
                }
            }
        }
 
        int res = f[s];
        out.println(res < INF ? res : -1);
        out.flush();
    }
    
    // mergeAlt merges the candidate set alt[u] (of a child) into alt[v] (of the parent),
    // taking into account the mod offsets.
    static void mergeAlt(int v, int u, int childMod) {
        // We want to merge the smaller set into the larger one.
        if(alt[v].size() >= alt[u].size()){
            int diff = childMod - modVal[v];
            for (Pair p : new ArrayList<>(alt[u])) {
                alt[v].add(new Pair(p.first + diff, p.second));
            }
        } else {
            int diff = modVal[v] - childMod;
            for (Pair p : new ArrayList<>(alt[v])) {
                alt[u].add(new Pair(p.first + diff, p.second));
            }
            alt[v] = alt[u];
            modVal[v] = childMod;
        }
    }
 
    // Standard DSU "find" with path compression.
    static int findDSU(int x) {
        if(dsuParent[x] != x)
            dsuParent[x] = findDSU(dsuParent[x]);
        return dsuParent[x];
    }
 
    // Standard DSU "union" by rank.
    static void unionDSU(int a, int b) {
        int ra = findDSU(a);
        int rb = findDSU(b);
        if(ra == rb) return;
        if(dsuRank[ra] > dsuRank[rb]){
            dsuParent[rb] = ra;
        } else {
            dsuParent[ra] = rb;
            if(dsuRank[ra] == dsuRank[rb])
                dsuRank[rb]++;
        }
    }
    
    // A simple pair class with two integers.
    static class Pair implements Comparable<Pair> {
        int first, second;
        public Pair(int first, int second) {
            this.first = first;
            this.second = second;
        }
        public int compareTo(Pair o) {
            if (this.first != o.first)
                return Integer.compare(this.first, o.first);
            return Integer.compare(this.second, o.second);
        }
        @Override
        public boolean equals(Object obj) {
            if (this == obj) return true;
            if (obj == null || getClass() != obj.getClass()) return false;
            Pair other = (Pair) obj;
            return first == other.first && second == other.second;
        }
        @Override
        public int hashCode() {
            return Objects.hash(first, second);
        }
    }
    
    // FastReader for competitive programming.
    static class FastReader {
        BufferedReader br;
        StringTokenizer st;
        public FastReader() {
            br = new BufferedReader(new InputStreamReader(System.in));
        }
        String next() throws IOException {
            while (st == null || !st.hasMoreTokens()){
                String line = br.readLine();
                if(line == null) return null;
                st = new StringTokenizer(line);
            }
            return st.nextToken();
        }
        int nextInt() throws IOException {
            return Integer.parseInt(next());
        }
        long nextLong() throws IOException {
            return Long.parseLong(next());
        }
        String nextLine() throws IOException {
            return br.readLine();
        }
    }
}
