#include <iostream>
#include <vector>
#include <queue>
#include <tuple>
#include <algorithm>
#include <cassert>
#include <map>
#include <cmath>
using namespace std;

const int MAX_H = 1600;
const int MAX_N = 30;
const int MAX_DIST = MAX_N * MAX_N + 2*MAX_N;

int n;
int h[MAX_N][MAX_N];
int distances[2 * MAX_DIST + 1][2 * MAX_DIST + 1];
vector<pair<int,int>> bfs_q[MAX_DIST * MAX_H];

void reset_arrays() {
    for (int i = 0; i < 2 * MAX_DIST + 1; ++i)
        for (int j = 0; j < 2 * MAX_DIST + 1; ++j)
            distances[i][j] = 1e8;

    for (int d = 0; d < MAX_DIST * MAX_H; d++) 
        bfs_q[d].clear();
}

// Directions for moving in the grid
int dx[4] = {1, -1, 0, 0};
int dy[4] = {0, 0, 1, -1};

vector<tuple<int, int, int>> find_short_cycles(int x0, int y0) {
    reset_arrays();

    bfs_q[0].push_back({x0, y0});
    distances[x0][y0] = 0;
    
    for (int d = 0; d < MAX_DIST * MAX_H; d++) {
        for (const auto& [x, y]: bfs_q[d]) {
            if (distances[x][y] < d) continue;
            
            for (int _ = 0; _ < 4; _++) {
                int x2 = x + dx[_], y2 = y + dy[_];
                int d2 = d + 1 + abs(h[x%n][y%n] - h[x2%n][y2%n]);
                if (abs(x0-x2) + abs(y0-y2) <= n*n
                        and distances[x2][y2] > d2) {
                    distances[x2][y2] = d2;
                    bfs_q[d2].push_back({x2, y2});
                }
            }
        }
    }

    // Collect results for shifts divisible by n.
    vector<tuple<int, int, int>> results;
    for (int a = -n*n; a <= n*n; a += n) {
        for (int b = -n*n; b <= n*n; b += n) {
            int d = distances[x0 + a][y0 + b];
            results.emplace_back(a, b, distances[x0 + a][y0 + b]);
        }
    }
    return results;
}


struct pt {
    double first, second;
    int dx, dy;
};

double norm2(pt P) {
    return pow(P.first, 2) + pow(P.second, 2);
}

bool operator<(pt A, pt B) {
    return (A.first == B.first) ? (A.second < B.second) : (A.first < B.first);
}

// Helper function to compute cross product
double cross(pt O, pt A, pt B) {
    return (A.first - O.first) * (B.second - O.second) - (A.second - O.second) * (B.first - O.first);
}

vector<pt> compute_convex_hull(vector<pt> points) {
    sort(points.begin(), points.end());
    vector<pt> hull;

    // Build lower hull
    for (const auto& p : points) {
        while (hull.size() >= 2 && cross(hull[hull.size() - 2], hull[hull.size() - 1], p) <= 0) {
            hull.pop_back();
        }
        hull.push_back(p);
    }

    // Build upper hull
    int lower_hull_size = hull.size();
    for (auto it = points.rbegin(); it != points.rend(); ++it) {
        while (hull.size() > lower_hull_size && cross(hull[hull.size() - 2], hull[hull.size() - 1], *it) <= 0) {
            hull.pop_back();
        }
        hull.push_back(*it);
    }

    hull.pop_back(); // Remove duplicate point
    return hull;
}

int main() {
    cin >> n;

    for (int i = 0; i < n; i++) for (int j = 0; j < n; j++) 
        cin >> h[i][j];


    map<pair<int,int>, pair<double, int>> dir2cost;
    auto process_short_cycles = [&](vector<tuple<int,int,int>> results) {
        for (const auto& [na, nb, d] : results) {
            if (d == 0) continue;
            if (d > 9e7) continue;
            double _d = d;
            
            int g = __gcd(abs(na), abs(nb));
            pair<int,int> pp = {na/g, nb/g};
            
            if (dir2cost.count(pp) == 0) {
                dir2cost[pp] = {_d / g, g};
            } else {
                auto curr = dir2cost[pp];
                if (curr.first >= _d / g + 1e-6)
                    dir2cost[pp] = {_d / g, g};
                if (curr.first >= _d / g - 1e-6 and curr.second > g)
                    dir2cost[pp] = {_d / g, g};
            }
        }
    };
    
    // Iterate over all cells where one coordinate is 0
    for (int i = 0; i < n; i++) {
        auto results = find_short_cycles(MAX_DIST + i, MAX_DIST);
        process_short_cycles(results);
    }
    for (int i = 1; i < n; i++) {
        auto results = find_short_cycles(MAX_DIST, MAX_DIST + i);
        process_short_cycles(results);
    }
    
    vector<pt> points; // To store points (x/d, y/d)
    for (auto dircc: dir2cost) {
        int x = dircc.first.first, y = dircc.first.second;
        double d = dircc.second.first;
        double g = dircc.second.second;
        pt P;
        P.first = x / d, P.second = y / d;
        P.dx = (x * g) / n, P.dy = (y * g) / n;
        points.push_back(P); 
    }
    
    // Compute convex hull of points
    vector<pt> convex_hull = compute_convex_hull(points);
    int ch = ssize(convex_hull);
    
    double area = 0;
    for (int i = 0; i < ch; i++) {
        pt A = convex_hull[i];
        pt B = convex_hull[(i + 1) % ch];
        area += cross({0, 0}, A, B);
    }
    area = abs(area / 2);

    
    // Remove unnecessary points.
    vector<pt> convex_hull2;
    for (int i = 0; i < ch; i++) {
        pt A = convex_hull[(i + ch-1) % ch];
        pt B = convex_hull[i];
        pt C = convex_hull[(i + 1) % ch];
        if (abs(cross(A, B, C)) > 1e-9 * area) {
            convex_hull2.push_back(B);
        }
    }
    convex_hull = convex_hull2;
    ch = ssize(convex_hull);
    
    clog << "Number of vertices in the convex hull: " << ch << endl;
    assert(ch % 2 == 0);
    clog << "List of (a, b) with b >= 0, where (an, bn)  is a fundamental cycle." << endl;
    for (int i = 0; i < ch/2; i++) {
        pt P = convex_hull[i];
        pt Q = convex_hull[i + ch/2];
        assert(P.dx == -Q.dx and P.dy == -Q.dy);
        clog << Q.dx << " " << Q.dy << endl;
    }
    
    double R = 1e20;
    cout.precision(10);
    cout << area * R*R << endl;
}
