#include <bits/stdc++.h>
using namespace std;

/**
 * Sums the elements of a vector<long long>.
 */
long long sumVector(const vector<long long>& arr) {
    long long s = 0;
    for (auto x : arr) s += x;
    return s;
}

/**
 * Attempt to build statue coordinates for the given problem.
 *
 * @param n     Number of statues.
 * @param a,b   Final statue coordinates.
 * @param d     Distances array of length n-1.
 *
 * @return A vector of (x,y) of length n if successful;
 *         an empty vector if no arrangement is possible.
 */
vector<pair<long long,long long>> tryBuildArrangement(
    int n,
    long long a,
    long long b,
    const vector<long long> &d
) {
    // We'll implement a direct "segment by segment" approach or any
    // method that either finds a valid arrangement or fails.

    // For the example, let's do a simple method:
    // We'll keep track of the current coordinate (cx,cy), start from (0,0).
    // For i in [1..n-1], we want statue i+1 at distance d[i-1] from (cx,cy).
    // The last one (i=n-1) must be exactly (a,b) if it fits the distance.
    //
    // If we can't place a valid coordinate for some statue, we return {}.

    // We'll store the statue positions in coords[].
    vector<pair<long long,long long>> coords(n);
    coords[0] = {0, 0}; // statue #1
    long long cx = 0, cy = 0;

    // A small helper to check if we can still finish from (x,y)
    // with segments d[i], d[i+1], ...
    // (We won't try an exhaustive search for large n; this is a simplified logic.)
    auto canReachEnd = [&](long long x, long long y, int segIndex) {
        // We have segments d[segIndex], ..., d[n-2], then must land on (a,b).
        // If segIndex == n-1, that means we are placing the last statue => check (x,y)==(a,b).
        // Otherwise, let's do the leftover distance logic:
        long long remainSum = 0;
        for (int k = segIndex; k < n - 1; k++) {
            remainSum += d[k];
        }
        long long direct = llabs(a - x) + llabs(b - y);
        if (remainSum < direct) return false;
        return ((remainSum - direct) % 2 == 0);
    };

    for (int i = 0; i < n - 1; i++) {
        long long distNeeded = d[i];

        // If this is the last segment, we must place statue n at (a,b).
        if (i == n - 2) {
            long long direct = llabs(a - cx) + llabs(b - cy);
            if (direct == distNeeded) {
                // Good. Place statue #n at (a,b)
                coords[i+1] = {a,b};
                cx = a; cy = b;
            }
            else {
                // No way to place the last statue with the correct distance
                return {}; // fail
            }
        }
        else {
            // We try to find any (nx,ny) with manhattanDist((cx,cy),(nx,ny))=distNeeded
            // that still allows finishing the arrangement.
            bool placed = false;

            // We'll do a small attempt: for large d[i], enumerating from 0..d[i] is impossible.
            // Instead, let's try just a few "big leaps" with sign combos:
            // e.g. put everything into x, or everything into y, or half split, etc.

            // We'll define a small set of (useX, useY) pairs that add up to distNeeded in absolute value:
            // For instance, we can do (distNeeded,0), (0,distNeeded), or partial splits around sqrt(distNeeded).
            // Then try sign combos. This won't find every possibility, but often it finds a feasible one
            // if it exists, especially with large leftover.
            vector<long long> trials;
            // Always consider 0, distNeeded
            trials.push_back(0);
            trials.push_back(distNeeded);
            // Maybe consider ~half
            long long half = distNeeded / 2;
            trials.push_back(half);
            long long quarter = distNeeded / 4;
            trials.push_back(quarter);
            long long threeQ = distNeeded - quarter;
            trials.push_back(threeQ);
            // clean up duplicates if any
            sort(trials.begin(), trials.end());
            trials.erase(unique(trials.begin(), trials.end()), trials.end());

            for (long long useX : trials) {
                long long useY = distNeeded - useX;
                // Because useX+useY=distNeeded => |useX|+|useY|=distNeeded if both are nonnegative.
                // But we also want sign combos: (±useX, ±useY).
                // We'll try 4 sign combos:
                vector<pair<long long,long long>> candidates;
                candidates.push_back({cx + useX, cy + useY});
                candidates.push_back({cx + useX, cy - useY});
                candidates.push_back({cx - useX, cy + useY});
                candidates.push_back({cx - useX, cy - useY});

                for (auto &cand : candidates) {
                    long long nx = cand.first;
                    long long ny = cand.second;
                    long long checkDist = llabs(nx - cx) + llabs(ny - cy);
                    if (checkDist == distNeeded) {
                        // Check if we can eventually finish from (nx,ny).
                        if (canReachEnd(nx, ny, i+1)) {
                            // place statue i+1
                            coords[i+1] = {nx, ny};
                            cx = nx; cy = ny;
                            placed = true;
                            break;
                        }
                    }
                }
                if (placed) break;
            }

            if (!placed) {
                return {}; // failed to place statue i+1
            }
        }
    }

    // At this point we should have placed all 3 statues.
    // Double-check the last statue is indeed (a,b):
    if (coords[n-1].first != a || coords[n-1].second != b) {
        return {};
    }

    // Final verify each distance
    for (int i = 0; i < n - 1; i++) {
        long long dd = llabs(coords[i+1].first - coords[i].first)
                     + llabs(coords[i+1].second - coords[i].second);
        if (dd != d[i]) {
            return {};
        }
    }
    return coords;
}


int main(){
    ios::sync_with_stdio(false);
    cin.tie(nullptr);

    int n;
    cin >> n;
    long long a,b;
    cin >> a >> b;
    vector<long long> d(n-1);
    for (int i = 0; i < n-1; i++){
        cin >> d[i];
    }

    // First, do the quick leftover check. If it fails => print NO immediately.
    long long S = sumVector(d);
    long long needed = a + b; // direct distance from (0,0) to (a,b)
    if (S < needed || ((S - needed) % 2LL != 0)) {
        cout << "NO\n";
        return 0;
    }

    // Next, try to build an arrangement. If it fails, we also print NO.
    vector<pair<long long, long long>> arrangement
        = tryBuildArrangement(n, a, b, d);

    if (arrangement.empty()) {
        // Means the function concluded there's no valid way to place them.
        cout << "NO\n";
        return 0;
    }

    // Otherwise, success. Print YES + the coordinates.
    cout << "YES\n";
    for (auto &coord : arrangement) {
        cout << coord.first << " " << coord.second << "\n";
    }
    return 0;
}
