#include <bits/stdc++.h>
using namespace std;

/**
 * Manhattan distance between (x1, y1) and (x2, y2).
 */
static long long manhattanDist(long long x1, long long y1,
                               long long x2, long long y2)
{
    return llabs(x1 - x2) + llabs(y1 - y2);
}

/**
 * Sum of all elements in a vector<long long>.
 */
static long long sumVector(const vector<long long>& arr)
{
    long long s = 0;
    for (auto x : arr) s += x;
    return s;
}

/**
 * Check if a leftover distance is feasible from (0,0) to (X,Y)
 * using 'remainDist' total distance left.
 * We require remainDist >= manhattanDist( (0,0), (X,Y) )
 * and same parity.
 */
bool canReachFromOrigin(long long X, long long Y, long long remainDist)
{
    long long direct = llabs(X) + llabs(Y);
    if (remainDist < direct) return false;
    // parity must match: (remainDist - direct) should be even
    return (((remainDist - direct) & 1LL) == 0);
}

/**
 * Try to place statue (x_i, y_i) given that statue (x_{i+1}, y_{i+1})
 * is already known (the "end"). We want:
 *   manhattanDist( (x_i,y_i), (endX,endY) ) = segDist  (i.e., d_i)
 * and we also want that it's still possible to place the earlier statues
 * so that statue #1 = (0,0). That means:
 *   canReachFromOrigin( x_i, y_i, leftoverDist )
 * where leftoverDist = sum of d_1..d_{i-1}.
 *
 * We'll try up to 2 candidate "residual distances" from (0,0) -> (x_i,y_i)
 * in the feasible intersection, and for each candidate we try
 * up to 16 sign combinations for the pair of diamond equations:
 *   |x_i - endX| + |y_i - endY| = segDist
 *   |x_i|      + |y_i|         = candidateResidual
 * to see if we can solve for x_i,y_i as integers.
 *
 * If successful, we set (bestX, bestY) to a solution and return true.
 * Otherwise, return false.
 */
bool findPrevStatue(int i,
                    long long segDist,        // d[i]
                    long long endX, long long endY, // statue i+1
                    long long leftoverDist,   // sum(d[1..i-1]) for backward
                    long long &bestX, long long &bestY)
{
    // Let directEnd = manhattanDist((0,0),(endX,endY)) be some reference
    // but we actually care about from (x_i,y_i) to (endX,endY) = segDist,
    // and from (0,0) to (x_i,y_i) <= leftoverDist in a feasible (parity) way.

    // We know by triangle inequality for (0,0)->(x_i,y_i)->(endX,endY):
    //   leftoverDist + segDist >= manhattanDist((0,0),(endX,endY))
    // But we must find the "residual distance" r = manhattanDist((x_i,y_i),(0,0)).
    // By triangle inequality with path (0,0)->(endX,endY)->(x_i,y_i), we get
    //   r in [ | segDist - directEnd |, segDist + directEnd ]
    // but also  r <= leftoverDist  and must satisfy parity with leftoverDist.

    long long directEnd = manhattanDist(0,0, endX, endY);

    // Lower and upper possible residual from (0,0) to (x_i,y_i)
    long long low = llabs(segDist - directEnd);
    long long high = segDist + directEnd;

    // Also clamp to [0, leftoverDist]
    if (low < 0) low = 0; // typically won't happen if segDist>directEnd
    if (high > leftoverDist) high = leftoverDist;

    // If low > high, no feasible range
    if (low > high) return false;

    // We also need the parity condition:
    // (r - directOrigin) mod 2 == 0 <=> (leftoverDist - r) is even
    // But actually we just want r itself to satisfy (leftoverDist - r) even.
    // => r % 2 == leftoverDist % 2
    long long neededParity = leftoverDist & 1LL;

    // We'll test up to 2 boundary values for r in [low..high] that match needed parity.
    // Because if there's a solution in that interval for r,
    // there should also be one at either the low or high boundary (with correct parity),
    // by Manhattan geometry arguments. So we only test `r = boundary` or `r = boundary +/- 1`.
    // Concretely, we define up to 2 candidates: rCandidate1, rCandidate2.
    // If the range is large, we won't iterate everything—just try the edges.
    vector<long long> candidates;
    // First candidate: the smallest r >= low with (r % 2 == neededParity)
    long long r1 = ( (low & 1LL) == neededParity ? low : low+1 );
    if (r1 <= high) candidates.push_back(r1);

    // Second candidate: the largest r <= high with (r % 2 == neededParity)
    long long r2 = ( (high & 1LL) == neededParity ? high : high-1 );
    if (r2 >= low) candidates.push_back(r2);

    // Remove duplicates if r1 == r2
    sort(candidates.begin(), candidates.end());
    candidates.erase(unique(candidates.begin(), candidates.end()), candidates.end());

    // For each candidate residual 'rCand', we want a point (x_i,y_i)
    // satisfying both diamond equations:
    //   D1: |x_i - endX| + |y_i - endY| = segDist
    //   D2: |x_i| + |y_i| = rCand
    // We'll do up to 16 sign combos, i.e. for D1 we choose signs for (x_i - endX),
    // for D2 we choose signs for x_i, y_i. Actually, each diamond can be in 4 sign forms:
    //   (x_i - endX)*sign1 + (y_i - endY)*sign2 = segDist
    //   x_i*sign3 + y_i*sign4 = rCand
    // Then solve the 2 linear equations in integers.

    auto tryAllSignCombos = [&](long long rCand)->bool {
        // We'll produce sign patterns for each diamond. Each diamond has 4 forms:
        //   (x - E)*(±1) + (y - F)*(±1) = segDist
        //   x*(±1) + y*(±1) = rCand
        // We'll systematically solve for (x,y).

        const long long E = endX, F = endY; // "end" coords
        // We'll define all sign combos as arrays of +1 or -1.
        vector<array<int,4>> combos;
        int signs[2] = {+1, -1};
        for (int s1 : signs) {
            for (int s2 : signs) {
                for (int s3 : signs) {
                    for (int s4 : signs) {
                        combos.push_back({s1,s2,s3,s4});
                    }
                }
            }
        }

        // We'll try each combo, solve the linear system:
        //   (x - E)*s1 + (y - F)*s2 = segDist   -- (1)
        //   (x)*s3 + (y)*s4 = rCand            -- (2)
        // That is:
        //   s1*x - s1*E + s2*y - s2*F = segDist
        //   s3*x + s4*y = rCand
        // We'll rewrite:
        //   s1*x + s2*y = segDist + s1*E + s2*F          -- eqA
        //   s3*x + s4*y = rCand                         -- eqB

        for (auto &c : combos) {
            int s1 = c[0], s2 = c[1], s3 = c[2], s4 = c[3];
            long long A = segDist + s1*E + s2*F; // right-hand side for eqA

            // eqA: s1*x + s2*y = A
            // eqB: s3*x + s4*y = rCand
            // We'll solve in integers if possible. That’s a 2x2 linear system:

            //   [ s1  s2 ] [ x ] = [ A     ]
            //   [ s3  s4 ] [ y ]   [ rCand ]

            long long det = (long long)s1*s4 - (long long)s2*s3;
            if (det == 0) {
                // no unique integer solution for x,y
                continue;
            }
            // Solve using Cramer's rule:
            // x = (A*s4 - s2*rCand)/ det
            // y = (s1*rCand - A*s3)/ det
            long long x_num = A*s4 - (long long)s2*rCand;
            long long y_num = (long long)s1*rCand - A*s3;

            // We need x_num%det==0, y_num%det==0 for integer solutions
            if ((x_num % det) != 0 || (y_num % det) != 0) {
                continue; // not an integer solution
            }
            long long x_sol = x_num / det;
            long long y_sol = y_num / det;

            // Now check if indeed:
            // 1) |x_sol - E| + |y_sol - F| == segDist
            // 2) |x_sol| + |y_sol| == rCand
            // to confirm we didn't break sign assumptions. But if we derived it from eqA/eqB,
            // we still must confirm the absolute-value forms hold:

            long long check1 = manhattanDist(x_sol, y_sol, E, F);
            long long check2 = llabs(x_sol) + llabs(y_sol);
            if (check1 == segDist && check2 == rCand) {
                // We found a valid (x_sol, y_sol)! Return it
                bestX = x_sol;
                bestY = y_sol;
                return true;
            }
        }
        return false;
    };

    // Try each candidate residual rCand
    for (auto rCand : candidates) {
        // We'll see if we can find any integer (x_i,y_i) that meets both diamond constraints.
        if (tryAllSignCombos(rCand)) {
            // Found it => success
            return true;
        }
    }
    // None worked
    return false;
}

/**
 * Attempt to build statue positions from statue #n = (a,b) backwards to #1 = (0,0).
 * If successful, returns the vector of n coordinates in "reverse" order:
 *   ret[0]   = statue #n
 *   ret[1]   = statue #(n-1)
 *   ...
 *   ret[n-1] = statue #1
 *
 * If impossible, returns an empty vector.
 */
vector<pair<long long,long long>> buildArrangementBackward(
    int n, long long a, long long b,
    const vector<long long> &d)
{
    // We'll store the statue coords in reverse order.
    // reversedCoords[0] = (a,b) => statue #n
    // reversedCoords[i] => statue #(n-i)
    vector<pair<long long,long long>> reversedCoords(n);
    // Start with statue #n
    reversedCoords[0] = {a, b};

    // sumDistUpTo(i) = d[1] + d[2] + ... + d[i], 1-based indexing for convenience
    // We only need leftoverDist = sum(d[1..i-1]) for backward steps.
    // We'll precompute partial sums to avoid repeated loops:
    vector<long long> prefix(n);
    prefix[0] = 0;
    for (int i2 = 1; i2 < n; i2++) {
        prefix[i2] = prefix[i2-1] + d[i2-1];
    }
    // prefix[i] = sum of d[0..(i-1)] = sum(d1..d_i) in 1-based sense

    // Now for i from (n-1) down to 1:
    //  - we know statue #(i+1) = reversedCoords[n-1-(i+1)+1] = reversedCoords[n-i-2],
    //    but easier to do it with an index transformation.
    // We'll define a direct loop over segment index j = 0..(n-2),
    // but j in descending order:
    //   i = n-1-j => leftoverDist = prefix[i-1] = prefix[n-1-j-1], etc.

    // We'll be careful with indexing to keep it readable:

    long long curEndX = a, curEndY = b; // "end" is statue #(i+1)
    reversedCoords[0] = {curEndX, curEndY}; // statue #n

    for (int seg = n-2; seg >= 0; seg--) {
        // We are placing statue #(seg+1).
        // The distance must be d[seg].
        // leftoverDist = sum(d[0..seg-1]) = prefix[seg].
        long long segDist = d[seg];
        long long leftoverDist = prefix[seg];
        // Attempt to find (x_i,y_i)
        long long bestX, bestY;
        if (!findPrevStatue(seg, segDist, curEndX, curEndY, leftoverDist, bestX, bestY)) {
            // no arrangement
            return {};
        }
        // We got statue #(seg+1) = (bestX, bestY).
        reversedCoords[n-1-seg] = {bestX, bestY};
        // Update "end" for next iteration
        curEndX = bestX;
        curEndY = bestY;
    }

    // Now reversedCoords[n-1] should be statue #1,
    // which must be (0,0). Let's check:
    auto [x0,y0] = reversedCoords[n-1];
    if (x0 != 0 || y0 != 0) {
        // something is off => no arrangement
        return {};
    }

    return reversedCoords;
}


int main(){
    ios::sync_with_stdio(false);
    cin.tie(nullptr);

    int n;
    cin >> n;
    long long a, b;
    cin >> a >> b;

    vector<long long> d(n-1);
    for (int i = 0; i < n-1; i++){
        cin >> d[i];
    }

    // 1) Quick leftover feasibility check
    long long S = sumVector(d);      // total of all d[i]
    long long needed = a + b;        // direct dist from (0,0) to (a,b)
    if (S < needed || ((S - needed) & 1LL)) {
        // not feasible
        cout << "NO\n";
        return 0;
    }

    // 2) Try to build an arrangement (backwards).
    //    If it fails, we say "NO". If it succeeds, print "YES" + coords in forward order.
    auto reversedCoords = buildArrangementBackward(n, a, b, d);
    if (reversedCoords.empty()) {
        // no arrangement found
        cout << "NO\n";
        return 0;
    }

    // 3) We have reversedCoords[0..n-1], where reversedCoords[0] = statue #n,
    //    reversedCoords[n-1] = statue #1. We want them in forward order:
    //    statue #1, #2, ..., #n
    // so we reverse the vector:
    reverse(reversedCoords.begin(), reversedCoords.end());

    // 4) Final safety check: for each i, distance must be d[i].
    //    Also statue #1 = (0,0), statue #n = (a,b).
    if (reversedCoords[0].first != 0 || reversedCoords[0].second != 0) {
        cout << "NO\n";
        return 0;
    }
    if (reversedCoords[n-1].first != a || reversedCoords[n-1].second != b) {
        cout << "NO\n";
        return 0;
    }
    for (int i = 0; i < n-1; i++) {
        long long checkDist = manhattanDist(reversedCoords[i].first,
                                            reversedCoords[i].second,
                                            reversedCoords[i+1].first,
                                            reversedCoords[i+1].second);
        if (checkDist != d[i]) {
            // mismatch
            cout << "NO\n";
            return 0;
        }
    }

    // 5) If everything is good, print "YES" and the arrangement
    cout << "YES\n";
    for (int i = 0; i < n; i++) {
        cout << reversedCoords[i].first << " " << reversedCoords[i].second << "\n";
    }
    return 0;
}
